/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/



#ifndef AQDB_DB_BE_H
#define AQDB_DB_BE_H

#include <aqdatabase/aqdb_db.h>
#include <aqdatabase/aqdb_column.h>

#include <gwenhywfar/buffer.h>



typedef int GWENHYWFAR_CB(*AQDB_DB_OPEN_FN)(AQDB_DB *db, uint32_t flags);
typedef int GWENHYWFAR_CB(*AQDB_DB_CREATE_FN)(AQDB_DB *db, uint32_t flags);
typedef int GWENHYWFAR_CB(*AQDB_DB_CLOSE_FN)(AQDB_DB *db, uint32_t flags);

typedef int GWENHYWFAR_CB(*AQDB_DB_CHECK_FN)(AQDB_DB *db, GWEN_BUFFER *resultTextBuf);


/**
 * Open a table inside the database.
 * @return 0 if ok, error code otherwise
 * @param db database
 * @param aflags access flag, @see AQDB_ACCESS_FLAGS_ISWRITER
 * @param tname name of the table to open
 * @param flags open mode, permissions etc (@see AQDB_ACTION_FLAGS_READ)
 * @param pTableId pointer to a variable receiving the id of the open table on success
 */
typedef int GWENHYWFAR_CB(*AQDB_DB_OPENTABLE_FN)(AQDB_DB *db,
                                                 uint32_t aflags,
                                                 const char *tname,
                                                 uint32_t flags,
                                                 AQDB_ID *pTableId);

/**
 * Create a table inside the database.
 * It will not be open afterwards, so you will need to call @ref AQDB_DB_OpenTable to
 * access the table.
 * @return 0 if ok, error code otherwise
 * @param db database
 * @param tname name of the table to create
 * @param flags open mode, permissions etc (@see AQDB_ACTION_FLAGS_READ)
 * @param columns list of columns for the table
 */
typedef int GWENHYWFAR_CB(*AQDB_DB_CREATETABLE_FN)(AQDB_DB *db,
                                                   const char *tname,
                                                   uint32_t flags,
                                                   const AQDB_COLUMN_LIST *columns);
typedef int GWENHYWFAR_CB(*AQDB_DB_CLOSETABLE_FN)(AQDB_DB *db,
                                                  AQDB_ID tableId,
                                                  uint32_t flags);

typedef int GWENHYWFAR_CB(*AQDB_DB_GETTABLENAMELIST_FN)(AQDB_DB *db,
                                                        GWEN_STRINGLIST *tableNameList);


typedef int GWENHYWFAR_CB(*AQDB_DB_READOBJECT_FN)(AQDB_DB *db,
                                                  uint32_t aflags,
                                                  AQDB_ID tableId,
                                                  AQDB_ID objectId,
                                                  AQDB_OBJECT **pObject);
typedef int GWENHYWFAR_CB(*AQDB_DB_WRITEOBJECT_FN)(AQDB_DB *db,
                                                   const AQDB_OBJECT *object);

typedef int GWENHYWFAR_CB(*AQDB_DB_DELETEOBJECT_FN)(AQDB_DB *db,
                                                    AQDB_ID tableId,
                                                    AQDB_ID objectId);
typedef int GWENHYWFAR_CB(*AQDB_DB_ADDOBJECT_FN)(AQDB_DB *db,
                                                 AQDB_ID tableId,
                                                 AQDB_OBJECT *object);

typedef int GWENHYWFAR_CB(*AQDB_DB_BEGINEDIT_FN)(AQDB_DB *db, uint32_t flags, const char *user);
typedef int GWENHYWFAR_CB(*AQDB_DB_ENDEDIT_FN)(AQDB_DB *db, uint32_t flags);

typedef int GWENHYWFAR_CB(*AQDB_DB_QUERYSUBMIT_FN)(AQDB_DB *db,
                                                   uint32_t aflags,
                                                   AQDB_ID tableId,
                                                   AQDB_ID refQueryId,
                                                   const char *queryExpression,
                                                   uint32_t flags,
                                                   AQDB_ID *pQueryId);

typedef int GWENHYWFAR_CB(*AQDB_DB_QUERYGETFIRST_FN)(AQDB_DB *db,
                                                     AQDB_ID queryId,
                                                     AQDB_ID *pIdListPtr,
                                                     int *pIdListLen);

typedef int GWENHYWFAR_CB(*AQDB_DB_QUERYGETNEXT_FN)(AQDB_DB *db,
                                                    AQDB_ID queryId,
                                                    AQDB_ID *pIdListPtr,
                                                    int *pIdListLen);

typedef int GWENHYWFAR_CB(*AQDB_DB_QUERYCLOSE_FN)(AQDB_DB *db, AQDB_ID queryId);


typedef int GWENHYWFAR_CB(*AQDB_DB_GETROLLBACKSESSIONS_FN)(AQDB_DB *db,
                                                           uint32_t aflags,
                                                           AQDB_SESSION_DESCR_LIST *sl);
typedef int GWENHYWFAR_CB(*AQDB_DB_UNDOSESSION_FN)(AQDB_DB *db, AQDB_ID sid);


typedef AQDB_COLUMN_LIST *GWENHYWFAR_CB(*AQDB_DB_GETCOLUMNLIST_FN)(AQDB_DB *db,
                                                                   uint32_t aflags,
                                                                   AQDB_ID tableId);


AQDB_API
AQDB_DB *AQDB_DB_new(const char *url);


AQDB_API
AQDB_DB_OPEN_FN AQDB_DB_SetOpenFn(AQDB_DB *db, AQDB_DB_OPEN_FN f);

AQDB_API
AQDB_DB_CREATE_FN AQDB_DB_SetCreateFn(AQDB_DB *db, AQDB_DB_CREATE_FN f);

AQDB_API
AQDB_DB_CLOSE_FN AQDB_DB_SetCloseFn(AQDB_DB *db, AQDB_DB_CLOSE_FN f);

AQDB_API
AQDB_DB_CHECK_FN AQDB_DB_SetCheckFn(AQDB_DB *db, AQDB_DB_CHECK_FN f);

AQDB_API
AQDB_DB_OPENTABLE_FN AQDB_DB_SetOpenTableFn(AQDB_DB *db, AQDB_DB_OPENTABLE_FN f);

AQDB_API
AQDB_DB_CREATETABLE_FN AQDB_DB_SetCreateTableFn(AQDB_DB *db, AQDB_DB_CREATETABLE_FN f);

AQDB_API
AQDB_DB_CLOSETABLE_FN AQDB_DB_SetCloseTableFn(AQDB_DB *db, AQDB_DB_CLOSETABLE_FN f);

AQDB_API
AQDB_DB_GETTABLENAMELIST_FN AQDB_DB_SetGetTableNameListFn(AQDB_DB *db, AQDB_DB_GETTABLENAMELIST_FN f);

AQDB_API
AQDB_DB_READOBJECT_FN AQDB_DB_SetReadObjectFn(AQDB_DB *db, AQDB_DB_READOBJECT_FN f);

AQDB_API
AQDB_DB_WRITEOBJECT_FN AQDB_DB_SetWriteObjectFn(AQDB_DB *db, AQDB_DB_WRITEOBJECT_FN f);

AQDB_API
AQDB_DB_DELETEOBJECT_FN AQDB_DB_SetDeleteObjectFn(AQDB_DB *db, AQDB_DB_DELETEOBJECT_FN f);

AQDB_API
AQDB_DB_ADDOBJECT_FN AQDB_DB_SetAddObjectFn(AQDB_DB *db, AQDB_DB_ADDOBJECT_FN f);

AQDB_API
AQDB_DB_QUERYSUBMIT_FN AQDB_DB_SetQuerySubmitFn(AQDB_DB *db, AQDB_DB_QUERYSUBMIT_FN f);

AQDB_API
AQDB_DB_QUERYGETFIRST_FN AQDB_DB_SetQueryGetFirstFn(AQDB_DB *db, AQDB_DB_QUERYGETFIRST_FN f);

AQDB_API
AQDB_DB_QUERYGETNEXT_FN AQDB_DB_SetQueryGetNextFn(AQDB_DB *db, AQDB_DB_QUERYGETNEXT_FN f);

AQDB_API
AQDB_DB_QUERYCLOSE_FN AQDB_DB_SetQueryCloseFn(AQDB_DB *db, AQDB_DB_QUERYCLOSE_FN f);

AQDB_API
AQDB_DB_BEGINEDIT_FN AQDB_DB_SetBeginEditFn(AQDB_DB *db, AQDB_DB_BEGINEDIT_FN f);

AQDB_API
AQDB_DB_ENDEDIT_FN AQDB_DB_SetEndEditFn(AQDB_DB *db, AQDB_DB_ENDEDIT_FN f);


AQDB_API
AQDB_DB_GETROLLBACKSESSIONS_FN AQDB_DB_SetGetRollbackSessionsFn(AQDB_DB *db, AQDB_DB_GETROLLBACKSESSIONS_FN f);

AQDB_API
AQDB_DB_UNDOSESSION_FN AQDB_DB_SetUndoSessionFn(AQDB_DB *db, AQDB_DB_UNDOSESSION_FN f);

AQDB_API
AQDB_DB_GETCOLUMNLIST_FN AQDB_DB_SetGetColumnListFn(AQDB_DB *db, AQDB_DB_GETCOLUMNLIST_FN f);



AQDB_API AQDB_DB_OPEN_FN AQDB_DB_GetOpenFn(AQDB_DB *db);

AQDB_API AQDB_DB_CREATE_FN AQDB_DB_GetCreateFn(AQDB_DB *db);

AQDB_API AQDB_DB_CLOSE_FN AQDB_DB_GetCloseFn(AQDB_DB *db);

AQDB_API AQDB_DB_CHECK_FN AQDB_DB_GetCheckFn(AQDB_DB *db);

AQDB_API AQDB_DB_OPENTABLE_FN AQDB_DB_GetOpenTableFn(AQDB_DB *db);

AQDB_API AQDB_DB_CREATETABLE_FN AQDB_DB_GetCreateTableFn(AQDB_DB *db);

AQDB_API AQDB_DB_CLOSETABLE_FN AQDB_DB_GetCloseTableFn(AQDB_DB *db);

AQDB_API AQDB_DB_GETTABLENAMELIST_FN AQDB_DB_GetGetTableNameListFn(AQDB_DB *db);

AQDB_API AQDB_DB_READOBJECT_FN AQDB_DB_GetReadObjectFn(AQDB_DB *db);

AQDB_API AQDB_DB_WRITEOBJECT_FN AQDB_DB_GetWriteObjectFn(AQDB_DB *db);

AQDB_API AQDB_DB_DELETEOBJECT_FN AQDB_DB_GetDeleteObjectFn(AQDB_DB *db);

AQDB_API AQDB_DB_ADDOBJECT_FN AQDB_DB_GetAddObjectFn(AQDB_DB *db);

AQDB_API AQDB_DB_QUERYSUBMIT_FN AQDB_DB_GetQuerySubmitFn(AQDB_DB *db);

AQDB_API AQDB_DB_QUERYGETFIRST_FN AQDB_DB_GetQueryGetFirstFn(AQDB_DB *db);

AQDB_API AQDB_DB_QUERYGETNEXT_FN AQDB_DB_GetQueryGetNextFn(AQDB_DB *db);

AQDB_API AQDB_DB_QUERYCLOSE_FN AQDB_DB_GetQueryCloseFn(AQDB_DB *db);

AQDB_API AQDB_DB_BEGINEDIT_FN AQDB_DB_GetBeginEditFn(AQDB_DB *db);

AQDB_API AQDB_DB_ENDEDIT_FN AQDB_DB_GetEndEditFn(AQDB_DB *db);


AQDB_API AQDB_DB_GETROLLBACKSESSIONS_FN AQDB_DB_GetGetRollbackSessionsFn(AQDB_DB *db);

AQDB_API AQDB_DB_UNDOSESSION_FN AQDB_DB_GetUndoSessionFn(AQDB_DB *db);

AQDB_API AQDB_DB_GETCOLUMNLIST_FN AQDB_DB_GetGetColumnListFn(AQDB_DB *db);




AQDB_API
uint32_t AQDB_DB_GetCacheHintMaxSize(const AQDB_DB *db);

AQDB_API
uint32_t AQDB_DB_GetCacheHintMaxEntries(const AQDB_DB *db);



AQDB_API
int AQDB_DB_ReadObjectWithAccessFlags(AQDB_DB *db,
                                      uint32_t aflags,
                                      AQDB_ID tableId,
                                      AQDB_ID objectId,
                                      AQDB_OBJECT **pObject);

AQDB_API
int AQDB_DB_BeginEditWithUserName(AQDB_DB *db, uint32_t flags, const char *userName);


AQDB_API
int AQDB_DB_QuerySubmitWithAccessFlags(AQDB_DB *db,
                                       uint32_t accessFlags,
                                       AQDB_ID tableId,
                                       AQDB_ID refQueryId,
                                       const char *queryExpression,
                                       uint32_t flags,
                                       AQDB_ID *pQueryId);



AQDB_API
int AQDB_DB_CacheGetObject(AQDB_DB *db,
                           AQDB_ID tableId,
                           AQDB_ID objectId,
                           AQDB_OBJECT **pObject);

AQDB_API
int AQDB_DB_CacheSetObject(AQDB_DB *db, const AQDB_OBJECT *object, uint32_t osize);

AQDB_API
void AQDB_DB_CachePurgeObject(AQDB_DB *db, AQDB_ID tableId, AQDB_ID objectId);




#endif






