/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifndef AQDB_OBJECT_H
#define AQDB_OBJECT_H

#include <aqdatabase/aqdb.h>
#include <aqdatabase/aqdb_value.h>

#include <gwenhywfar/list1.h>
#include <gwenhywfar/list2.h>
#include <gwenhywfar/buffer.h>
#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/gwendate.h>

#ifdef __cplusplus
extern "C" {
#endif


typedef struct AQDB_OBJECT AQDB_OBJECT;

GWEN_LIST_FUNCTION_LIB_DEFS(AQDB_OBJECT, AQDB_Object, AQDB_API)
GWEN_LIST2_FUNCTION_LIB_DEFS(AQDB_OBJECT, AQDB_Object, AQDB_API)


AQDB_API
AQDB_OBJECT *AQDB_Object_new(AQDB_ID tableId,
                             AQDB_ID id,
                             uint32_t nfields);


AQDB_API
void AQDB_Object_Attach(AQDB_OBJECT *o);

AQDB_API
void AQDB_Object_free(AQDB_OBJECT *o);


AQDB_API
AQDB_ID AQDB_Object_GetObjectId(const AQDB_OBJECT *o);

AQDB_API
AQDB_ID AQDB_Object_GetTableId(const AQDB_OBJECT *o);

AQDB_API
uint32_t AQDB_Object_GetFieldCount(const AQDB_OBJECT *o);



AQDB_API
int AQDB_Object_GetField(const AQDB_OBJECT *o,
                         uint32_t idx,
                         uint8_t **pData,
                         uint32_t *pLen);

AQDB_API
int AQDB_Object_SetField(AQDB_OBJECT *o,
                         uint32_t idx,
                         const uint8_t *data,
                         uint32_t len);



/** @name Convenience Functions
 *
 */
/*@{*/
AQDB_API
int AQDB_Object_GetFieldInt64(const AQDB_OBJECT *o,
                              uint32_t idx,
                              int64_t *pVal);

AQDB_API
int AQDB_Object_SetFieldInt64(AQDB_OBJECT *o,
                              uint32_t idx,
                              int64_t val);

AQDB_API
int AQDB_Object_GetFieldInt32(const AQDB_OBJECT *o,
                              uint32_t idx,
                              int32_t *pVal);

AQDB_API
int AQDB_Object_SetFieldInt32(AQDB_OBJECT *o,
                              uint32_t idx,
                              int32_t val);

AQDB_API
int AQDB_Object_GetFieldString(const AQDB_OBJECT *o,
                               uint32_t idx,
                               const char **pVal);

AQDB_API
int AQDB_Object_SetFieldString(AQDB_OBJECT *o,
                               uint32_t idx,
                               const char *s);

AQDB_API
int AQDB_Object_GetFieldDate(AQDB_OBJECT *o,
                             uint32_t idx,
                             const GWEN_DATE **pVal);

AQDB_API
int AQDB_Object_SetFieldDate(AQDB_OBJECT *o,
                             uint32_t idx,
                             const GWEN_DATE *val);

AQDB_API
int AQDB_Object_GetFieldTimestamp(AQDB_OBJECT *o,
                                  uint32_t idx,
                                  const GWEN_TIME **pVal);

AQDB_API
int AQDB_Object_SetFieldTimestamp(AQDB_OBJECT *o,
                                  uint32_t idx,
                                  const GWEN_TIME *val);


AQDB_API
int AQDB_Object_GetFieldValue(AQDB_OBJECT *o,
                              uint32_t idx,
                              const AQDB_VALUE **pVal);

AQDB_API
int AQDB_Object_SetFieldValue(AQDB_OBJECT *o,
                              uint32_t idx,
                              const AQDB_VALUE *val);


/**
 * Set a field value from string according to field type.
 *
 * @return 0 if okay, error code otherwise
 * @param o object to which the value is to be set
 * @param idx number of the field to set (starting with "0")
 * @param t data type of the field (data will be transformed to that type accordingly)
 * @param data string to convert to the given data type and then be stored
 * @param modifier optional modifier (template for dates and times, "hex"/"base64" for bin)
 */
AQDB_API int AQDB_Object_SetFieldFromStringByType(AQDB_OBJECT *o,
                                                  uint32_t idx,
                                                  AQDB_DATA_TYPE t,
                                                  const char *data,
                                                  const char *modifier);

/**
 * Get a field as a string according to field type.
 *
 * @return 0 if okay, error code otherwise
 * @param o object for which the value is to be read
 * @param idx number of the field to get (starting with "0")
 * @param t data type of the field (data will be transformed to that type accordingly)
 * @param modifier optional modifier (template for dates and times, "hex"/"base64" for bin, precision for amount)
 * @param destBuf buffer to receive the data

 */
AQDB_API int AQDB_Object_GetFieldAsStringByType(AQDB_OBJECT *o,
                                                uint32_t idx,
                                                AQDB_DATA_TYPE t,
                                                const char *modifier,
                                                GWEN_BUFFER *destBuf);
/*@}*/


void AQDB_Object_Dump(const AQDB_OBJECT *o, int indent);


#ifdef __cplusplus
}
#endif


#endif

