/****************************************************************************
 * This file is part of the project AqDataBase.
 * AqDatabase (c) by 2020 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifndef AQDB_USER_SESSION_H
#define AQDB_USER_SESSION_H

#include <aqdatabase/user/aqdb_userdb.h>
#include <aqdatabase/user/aqdb_usertable.h>

#include <gwenhywfar/types.h>
#include <gwenhywfar/inherit.h>


#define AQDB_USER_SESSION_MAX_TABLES  128
#define AQDB_USER_SESSION_MAX_QUERIES 128

#define AQDB_USER_SESSION_MAX_GROUPS  16


#ifdef __cplusplus
extern "C" {
#endif

typedef struct AQDB_USER_SESSION AQDB_USER_SESSION;
GWEN_INHERIT_FUNCTION_LIB_DEFS(AQDB_USER_SESSION, AQDB_API)



AQDB_USER_SESSION *AQDB_UserSession_new(AQDB_DB *database, const char *userName);
void AQDB_UserSession_free(AQDB_USER_SESSION *userSession);

const char *AQDB_UserSession_GetSessionName(const AQDB_USER_SESSION *userSession);
void AQDB_UserSession_SetSessionName(AQDB_USER_SESSION *userSession, const char *s);

const char *AQDB_UserSession_GetUserName(const AQDB_USER_SESSION *userSession);
void AQDB_UserSession_SetUserName(AQDB_USER_SESSION *userSession, const char *s);

AQDB_ID AQDB_UserSession_GetUserId(const AQDB_USER_SESSION *userSession);
void AQDB_UserSession_SetUserId(AQDB_USER_SESSION *userSession, AQDB_ID id);

int AQDB_UserSession_GetMaxNumGroupIds(const AQDB_USER_SESSION *userSession);
AQDB_ID AQDB_UserSession_GetGroupIdAt(const AQDB_USER_SESSION *userSession, int idx);
int AQDB_UserSession_SetGroupIdAt(AQDB_USER_SESSION *userSession, int idx, AQDB_ID id);
int AQDB_UserSession_AddGroupId(AQDB_USER_SESSION *userSession, AQDB_ID id);

int AQDB_UserSession_GetMaxNumOpenTables(const AQDB_USER_SESSION *userSession);
AQDB_USER_TABLE *AQDB_UserSession_GetOpenTableAt(const AQDB_USER_SESSION *userSession, int idx);
int AQDB_UserSession_SetOpenTableAt(AQDB_USER_SESSION *userSession, int idx, AQDB_USER_TABLE *userTable);
int AQDB_UserSession_AddOpenTable(AQDB_USER_SESSION *userSession, AQDB_USER_TABLE *userTable);

AQDB_ID AQDB_UserSession_GetMaxNumOpenQueryIds(const AQDB_USER_SESSION *userSession);
AQDB_ID AQDB_UserSession_GetOpenQueryIdAt(const AQDB_USER_SESSION *userSession, int idx);
int AQDB_UserSession_SetOpenQueryIdAt(AQDB_USER_SESSION *userSession, int idx, AQDB_ID id);
int AQDB_UserSession_AddOpenQueryId(AQDB_USER_SESSION *userSession, AQDB_ID id);


#ifdef __cplusplus
}
#endif


#endif

